package mood

import (
	"context"

	"github.com/kamva/mgm/v3"
	"go.mongodb.org/mongo-driver/bson"
)

type Mood struct {
	// DefaultModel adds _id, created_at and updated_at fields to the Model
	mgm.DefaultModel `bson:",inline"`
	Name             string `json:"name" bson:"name"`
}

func create(ctx context.Context, mood *Mood) (Mood, error) {
	coll := mgm.Coll(mood)

	err := coll.CreateWithCtx(ctx, mood)

	if err != nil {
		return Mood{}, err
	}

	return *mood, nil
}

func delete(ctx context.Context, id interface{}) error {
	mood := &Mood{}
	coll := mgm.Coll(mood)

	err := coll.FindByIDWithCtx(ctx, id, mood)
	if err != nil {
		return err
	}

	err = coll.DeleteWithCtx(ctx, mood)
	if err != nil {
		return err
	}

	return nil
}

func findAll(ctx context.Context) ([]Mood, error) {
	coll := mgm.Coll(&Mood{})

	result, err := coll.Find(ctx, bson.M{})
	if err != nil {
		return nil, err
	}

	mood := make([]Mood, 0)
	err = result.All(ctx, &mood)
	if err != nil {
		return nil, err
	}

	return mood, nil
}

func findById(ctx context.Context, id interface{}) (Mood, error) {
	mood := &Mood{}
	coll := mgm.Coll(mood)

	err := coll.FindByIDWithCtx(ctx, id, mood)
	if err != nil {
		return Mood{}, err
	}

	return *mood, nil
}

func update(ctx context.Context, id interface{}, newMood *Mood) (Mood, error) {
	mood := &Mood{}
	coll := mgm.Coll(mood)

	err := coll.FindByIDWithCtx(ctx, id, mood)
	if err != nil {
		return Mood{}, err
	}

	mood.Name = newMood.Name

	err = coll.UpdateWithCtx(ctx, mood)
	if err != nil {
		return Mood{}, err
	}

	return *mood, nil
}
