package routine_data

import (
	"encoding/json"
	"log"
	"net/http"

	"github.com/gorilla/mux"
)

type RoutineDataRouter struct{}

func NewRoutineDataRouter() *RoutineDataRouter {
	return &RoutineDataRouter{}
}

func (u *RoutineDataRouter) RegisterRoutes(r *mux.Router) {
	r.Path("/").Methods(http.MethodGet).HandlerFunc(u.GetAll)
	r.Path("/{id}").Methods(http.MethodGet).HandlerFunc(u.Get)
	r.PathPrefix("/").Methods(http.MethodPost).HandlerFunc(u.Post)
	r.PathPrefix("/{id}").Methods(http.MethodPut).HandlerFunc(u.Put)
	r.Path("/{id}").Methods(http.MethodDelete).HandlerFunc(u.Delete)
}

func (u *RoutineDataRouter) Get(rw http.ResponseWriter, r *http.Request) {
	vars := mux.Vars(r)

	routineData, err := findById(r.Context(), vars["id"])
	if err != nil {
		log.Printf("error finding routine data by id: %s", err)
		rw.WriteHeader(http.StatusInternalServerError)
		return
	}

	bytes, err := json.Marshal(routineData)
	if err != nil {
		log.Printf("error marshaling routine data: %s", err)
		rw.WriteHeader(http.StatusInternalServerError)
		return
	}

	rw.Header().Set("Content-Type", "application/json")
	rw.Write(bytes)
}

func (u *RoutineDataRouter) GetAll(rw http.ResponseWriter, r *http.Request) {
	routineData, err := findAll(r.Context())
	if err != nil {
		log.Printf("error finding routine data: %s", err)
		rw.WriteHeader(http.StatusInternalServerError)
		return
	}

	bytes, err := json.Marshal(routineData)
	if err != nil {
		log.Printf("error marshaling routine data: %s", err)
		rw.WriteHeader(http.StatusInternalServerError)
		return
	}

	rw.Header().Set("Content-Type", "application/json")
	rw.Write(bytes)
}

func (u *RoutineDataRouter) Post(rw http.ResponseWriter, r *http.Request) {
	var routineData RoutineData

	err := json.NewDecoder(r.Body).Decode(&routineData)
	if err != nil {
		log.Printf("error decoding routine data: %s", err)
		rw.WriteHeader(http.StatusInternalServerError)
		return
	}

	routineData, err = create(r.Context(), &routineData)
	if err != nil {
		log.Printf("error creating routine data: %s", err)
		rw.WriteHeader(http.StatusInternalServerError)
		return
	}

	bytes, err := json.Marshal(routineData)
	if err != nil {
		log.Printf("error marshaling routine data: %s", err)
		rw.WriteHeader(http.StatusInternalServerError)
		return
	}

	rw.Header().Set("Content-Type", "application/json")
	rw.Write(bytes)
}

func (u *RoutineDataRouter) Put(rw http.ResponseWriter, r *http.Request) {
	vars := mux.Vars(r)
	var routineData RoutineData

	err := json.NewDecoder(r.Body).Decode(&routineData)
	if err != nil {
		log.Printf("error decoding routine data: %s", err)
		rw.WriteHeader(http.StatusInternalServerError)
		return
	}

	routineData, err = update(r.Context(), vars["id"], &routineData)
	if err != nil {
		log.Printf("error creating routine data: %s", err)
		rw.WriteHeader(http.StatusInternalServerError)
		return
	}

	bytes, err := json.Marshal(routineData)
	if err != nil {
		log.Printf("error marshaling routine data: %s", err)
		rw.WriteHeader(http.StatusInternalServerError)
		return
	}

	rw.Header().Set("Content-Type", "application/json")
	rw.Write(bytes)
}

func (u *RoutineDataRouter) Delete(rw http.ResponseWriter, r *http.Request) {
	vars := mux.Vars(r)

	err := delete(r.Context(), vars["id"])
	if err != nil {
		log.Printf("error finding routine data by id: %s", err)
		rw.WriteHeader(http.StatusInternalServerError)
		return
	}

	rw.Header().Set("Content-Type", "application/json")
	rw.WriteHeader(http.StatusOK)
}
