package routine_data

import (
	"context"

	"github.com/kamva/mgm/v3"
	"go.mongodb.org/mongo-driver/bson"
)

type RoutineData struct {
	// DefaultModel adds _id, created_at and updated_at fields to the Model
	mgm.DefaultModel `bson:",inline"`
	UserID           interface{}      `json:"user_id" bson:"user_id"`
	RoutineID        interface{}      `json:"routine_id" bson:"routine_id"`
	Moods            []Mood           `json:"moods" bson:"moods"`
	AdditionalInfo   []AdditionalInfo `json:"additional_info" bson:"additional_info"`
	DayDescription   string           `json:"day_description" bson:"day_description"`
}

type Mood struct {
	Name   string `json:"name" bson:"name"`
	Rating int    `json:"rating" bson:"rating"`
}

type AdditionalInfo struct {
	Name   string `json:"name" bson:"name"`
	Rating int    `json:"rating" bson:"rating"`
}

func create(ctx context.Context, routineData *RoutineData) (RoutineData, error) {
	coll := mgm.Coll(routineData)

	err := coll.CreateWithCtx(ctx, routineData)

	if err != nil {
		return RoutineData{}, err
	}

	return *routineData, nil
}

func delete(ctx context.Context, id interface{}) error {
	routineData := &RoutineData{}
	coll := mgm.Coll(routineData)

	err := coll.FindByIDWithCtx(ctx, id, routineData)
	if err != nil {
		return err
	}

	err = coll.DeleteWithCtx(ctx, routineData)
	if err != nil {
		return err
	}

	return nil
}

func findAll(ctx context.Context) ([]RoutineData, error) {
	coll := mgm.Coll(&RoutineData{})

	result, err := coll.Find(ctx, bson.M{})
	if err != nil {
		return nil, err
	}

	routineData := make([]RoutineData, 0)
	err = result.All(ctx, &routineData)
	if err != nil {
		return nil, err
	}

	return routineData, nil
}

func findById(ctx context.Context, id interface{}) (RoutineData, error) {
	routineData := &RoutineData{}
	coll := mgm.Coll(routineData)

	err := coll.FindByIDWithCtx(ctx, id, routineData)
	if err != nil {
		return RoutineData{}, err
	}

	return *routineData, nil
}

func update(ctx context.Context, id interface{}, newRoutineData *RoutineData) (RoutineData, error) {
	routineData := &RoutineData{}
	coll := mgm.Coll(routineData)

	err := coll.FindByIDWithCtx(ctx, id, routineData)
	if err != nil {
		return RoutineData{}, err
	}

	routineData.UserID = newRoutineData.UserID
	routineData.RoutineID = newRoutineData.RoutineID
	routineData.Moods = newRoutineData.Moods
	routineData.AdditionalInfo = newRoutineData.AdditionalInfo
	routineData.DayDescription = newRoutineData.DayDescription

	err = coll.UpdateWithCtx(ctx, routineData)
	if err != nil {
		return RoutineData{}, err
	}

	return *routineData, nil
}
