package user

import (
	"context"

	"github.com/kamva/mgm/v3"
	"go.mongodb.org/mongo-driver/bson"
)

type User struct {
	// DefaultModel adds _id, created_at and updated_at fields to the Model
	mgm.DefaultModel `bson:",inline"`
	Email            string `json:"email" bson:"email"`
	FirstName        string `json:"first_name" bson:"first_name"`
	LastName         string `json:"last_name" bson:"last_name"`
	Password         string `json:"password" bson:"password"`
	SessionToken     string `json:"session_token" bson:"session_token"`
}

func create(ctx context.Context, user *User) (User, error) {
	coll := mgm.Coll(user)

	err := coll.CreateWithCtx(ctx, user)

	if err != nil {
		return User{}, err
	}

	return *user, nil
}

func delete(ctx context.Context, id interface{}) error {
	user := &User{}
	coll := mgm.Coll(user)

	err := coll.FindByIDWithCtx(ctx, id, user)
	if err != nil {
		return err
	}

	err = coll.DeleteWithCtx(ctx, user)
	if err != nil {
		return err
	}

	return nil
}

func findAll(ctx context.Context) ([]User, error) {
	coll := mgm.Coll(&User{})

	result, err := coll.Find(ctx, bson.M{})
	if err != nil {
		return nil, err
	}

	users := make([]User, 0)
	err = result.All(ctx, &users)
	if err != nil {
		return nil, err
	}

	return users, nil
}

func findById(ctx context.Context, id interface{}) (User, error) {
	user := &User{}
	coll := mgm.Coll(user)

	err := coll.FindByIDWithCtx(ctx, id, user)
	if err != nil {
		return User{}, err
	}

	return *user, nil
}

func update(ctx context.Context, id interface{}, newUser *User) (User, error) {
	user := &User{}
	coll := mgm.Coll(user)

	err := coll.FindByIDWithCtx(ctx, id, user)
	if err != nil {
		return User{}, err
	}

	user.Email = newUser.Email
	user.FirstName = newUser.FirstName
	user.LastName = newUser.LastName
	user.Password = newUser.Password
	user.SessionToken = newUser.SessionToken

	err = coll.UpdateWithCtx(ctx, user)
	if err != nil {
		return User{}, err
	}

	return *user, nil
}
